package com.soss.system.service.impl;

import com.soss.common.enums.CouponCategoryType;
import com.soss.common.exception.ServiceException;
import com.soss.common.utils.GenerateCode;
import com.soss.system.domain.Coupon;
import com.soss.system.domain.CouponCategory;
import com.soss.system.domain.CouponRule;
import com.soss.system.domain.po.CouponPo;
import com.soss.system.mapper.CouponCategoryMapper;
import com.soss.system.mapper.CouponMapper;
import com.soss.system.mapper.CouponRuleMapper;
import com.soss.system.mapper.CouponUserMapper;
import com.soss.system.service.ICouponService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.stream.Collectors;

/**
 * <p>
 * 优惠券 服务实现类
 * </p>
 *
 * @author caiyt
 * @since 2022-07-21
 */
@Service
public class CouponServiceImpl implements ICouponService {
    @Autowired
    private CouponCategoryMapper couponCategoryMapper;
    @Autowired
    private CouponMapper couponMapper;
    @Autowired
    private CouponRuleMapper couponRuleMapper;
    @Autowired
    private CouponUserMapper couponUserMapper;

    @Override
    @Transactional
    public void save(CouponPo couponPo) {
        Coupon coupon = new Coupon();
        CouponRule couponRule = new CouponRule();

        this.checkAndWrapperCoupon(couponPo, coupon, couponRule, true);
        couponRuleMapper.insertCouponRule(couponRule);
        coupon.setRuleId(couponRule.getId());
        couponMapper.insertCoupon(coupon);
    }

    @Override
    @Transactional
    public void update(CouponPo couponPo) {
        Assert.notNull(couponPo.getId(), "要修改的优惠券ID还未传递");
        Coupon coupon = couponMapper.selectCouponById(couponPo.getId());
        Assert.notNull(coupon, "未查询到匹配的优惠券记录[id=" + couponPo.getId() + "]");
        int couponReceiveCount = couponUserMapper.getCouponReceiveCount(coupon.getId());
        if (couponReceiveCount > 0) {
            throw new ServiceException("优惠券已被领取，无法编辑，只可下线");
        }
        CouponRule couponRule = couponRuleMapper.selectCouponRuleById(coupon.getRuleId());
        checkAndWrapperCoupon(couponPo, coupon, couponRule, false);
        couponRuleMapper.updateCouponRule(couponRule);
        couponMapper.updateCoupon(coupon);
    }

    private void checkAndWrapperCoupon(CouponPo couponPo, Coupon coupon, CouponRule couponRule, boolean saveFlag) {
        Assert.notNull(couponPo.getName(), "优惠券名称还未填写");
        Assert.notNull(couponPo.getCategoryId(), "优惠券类别还未选择");
        CouponCategory couponCategory = couponCategoryMapper.selectCouponCategoryById(couponPo.getCategoryId());
        Assert.notNull(couponCategory, "未查询到匹配的优惠券类别[id=" + couponPo.getCategoryId() + "]");
        String ruleName, ruleDesc; // 规则名称，规则描述
        if (CouponCategoryType.DEDUCTION.getType().equals(couponCategory.getType())) {
            Assert.notNull(couponPo.getPriceDiscount(), "满减价还未设置");
            Assert.notNull(couponPo.getPriceLimit(), "价格门槛还未设置");
            Assert.notNull(couponPo.getDaysLimit(), "用户可领取次数还未设置");
            ruleName = "立减" + couponPo.getPriceDiscount() + "元";
            if (couponPo.getPriceLimit().compareTo(BigDecimal.ZERO) == 0) {
                ruleDesc = "无门槛";
            } else {
                ruleDesc = "满" + couponPo.getPriceLimit() + "元可用";
            }
        } else if (CouponCategoryType.DISCOUNT.getType().equals(couponCategory.getType())) {
            Assert.notNull(couponPo.getPriceDiscount(), "打折数还未设置");
            Assert.notNull(couponPo.getOrderLimit(), "订单限制还未设置");
            ruleName = couponPo.getOrderLimit() + "折";
            ruleDesc = couponPo.getOrderLimit() ? "单杯" : "整单";
        } else {
            Assert.notNull(couponPo.getPriceLimit(), "价格门槛还未设置");
            Assert.notNull(couponPo.getOrderLimit(), "订单限制还未设置");
            ruleName = "免单";
            if (couponPo.getPriceLimit().compareTo(BigDecimal.ZERO) == 0) {
                ruleDesc = "无门槛";
            } else {
                ruleDesc = "满" + couponPo.getPriceLimit() + "元可用";
            }
        }
        Assert.notNull(couponPo.getUseStartTime(), "优惠券的绝对有效时间还未设置");
        Assert.notNull(couponPo.getUseEndTime(), "优惠券的绝对有效时间还未设置");
        Assert.isTrue(!CollectionUtils.isEmpty(couponPo.getCategoryIds())
                && !CollectionUtils.isEmpty(couponPo.getGoodsIds()), "商品范围还未设置");
        Assert.notNull(couponPo.getSendMsg(), "是否短信通知还未设置");
        Assert.isTrue(couponPo.getSendMsg() && couponPo.getMsgId() != null, "短信模板还未选择");

        coupon.setName(couponPo.getName());
        coupon.setSerialNo(GenerateCode.getCode("CP", "%06d"));
        coupon.setCategoryId(couponCategory.getId());
        coupon.setCategoryName(couponCategory.getName());
        coupon.setType(couponCategory.getType());
        LocalDateTime now = LocalDateTime.now();
        if (saveFlag) {
            coupon.setCreatedAt(now);
        }
        coupon.setUpdatedAt(now);

        BeanUtils.copyProperties(couponPo, couponRule, "name");
        couponRule.setName(ruleName);
        couponRule.setDesc(ruleDesc);
        couponRule.setCategoryIds(couponPo.getCategoryIds().stream().map(String::valueOf).collect(Collectors.joining(",")));
        couponRule.setGoodsIds(couponPo.getGoodsIds().stream().map(String::valueOf).collect(Collectors.joining(",")));
        couponRule.setProvince(String.join(",", couponPo.getProvince()));
        couponRule.setCity(String.join(",", couponPo.getCity()));
        couponRule.setArea(String.join(",", couponPo.getArea()));
        couponRule.setShopIds(couponPo.getShopIds().stream().map(String::valueOf).collect(Collectors.joining(",")));
        if (saveFlag) {
            couponRule.setCreatedAt(now);
        }
        couponRule.setUpdatedAt(now);
    }
}
